----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 19.04.2021 13:37:04
-- Design Name: 
-- Module Name: Test_Etage4_Memoire - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity Test_Etage4_Memoire is
--  Port ( );
end Test_Etage4_Memoire;

architecture Behavioral of Test_Etage4_Memoire is
    component Etage4_Memoire is
        Generic ( Nb_bits : Natural; -- Taille d'un mot binaire
                  Mem_size : Natural; -- Taille de la mémoire de donnees (nombre de mots binaires stockables)
                  Adresse_mem_size : Natural; -- Nombre de bits pour adresser la mémoire de donnees
                  Instruction_bus_size : Natural; -- Nombre de bits du bus d'instruction (Taille d'un code instruction)
                  Bits_Controle_LC : STD_LOGIC_VECTOR; -- Vecteur de bit controlant le Link Controler (cf LC.vhd)
                  Bits_Controle_MUX_IN : STD_LOGIC_VECTOR; -- Vecteur de bit controlant le multiplexer selectionnant A ou B comme adresse (cf MUX.vhd)
                  Bits_Controle_MUX_IN_EBP : STD_LOGIC_VECTOR; -- Vecteur de bit controlant le multiplexer selectionnant si on doit ajouter ou non EBP à l'adresse (cf MUX.vhd)
                  Bits_Controle_MUX_OUT : STD_LOGIC_VECTOR; -- Vecteur de bit controlant le multiplexer de sortie (cf MUX.vhd)
                  Code_Instruction_CALL : STD_LOGIC_VECTOR; -- Numéro de l'instruction CALL
                  Code_Instruction_RET : STD_LOGIC_VECTOR); -- Numéro de l'instruction RET
        Port ( CLK : in STD_LOGIC; -- Clock
               RST : in STD_LOGIC; -- Reset
               IN_A : in STD_LOGIC_VECTOR (Nb_bits - 1 downto 0); -- Entrée de l'opérande A
               IN_B : in STD_LOGIC_VECTOR (Nb_bits - 1 downto 0); -- Entrée de l'opérande B
               IN_Instruction : in STD_LOGIC_VECTOR (Instruction_bus_size - 1 downto 0); -- Entrée de l'instruction
               OUT_A : out STD_LOGIC_VECTOR (Nb_bits - 1 downto 0); -- Sortie de l'opérande A
               OUT_B : out STD_LOGIC_VECTOR (Nb_bits - 1 downto 0); -- Sortie de l'opérande B
               OUT_Instruction : out STD_LOGIC_VECTOR (Instruction_bus_size - 1 downto 0); -- Sortie de l'instruction
               OUT_AddrRetour : out STD_LOGIC_VECTOR (Nb_bits - 1 downto 0)); -- Sortie de l'adresse de retour vers l'étage 1
    end component;
    
    signal my_CLK : STD_LOGIC := '0';
    signal my_RST : STD_LOGIC := '1';
    signal my_IN_A : STD_LOGIC_VECTOR (7 downto 0) := (others => '0');
    signal my_IN_B : STD_LOGIC_VECTOR (7 downto 0) := (others => '0');
    signal my_IN_Instruction : STD_LOGIC_VECTOR (4 downto 0) := (others => '0');
    signal my_OUT_A : STD_LOGIC_VECTOR (7 downto 0) := (others => '0');
    signal my_OUT_B : STD_LOGIC_VECTOR (7 downto 0) := (others => '0');
    signal my_OUT_Instruction : STD_LOGIC_VECTOR (4 downto 0) := (others => '0');
    signal my_OUT_AddrRetour : STD_LOGIC_VECTOR (7 downto 0) := (others => '0');
    
    constant Bits_Controle_LC : STD_LOGIC_VECTOR (31 downto 0) := "1111111111" & "1111111001011111111111";
    constant Bits_Controle_MUX_IN : STD_LOGIC_VECTOR (31 downto 0) := "1111111111" & "1101111110101111111111";
    constant Bits_Controle_MUX_IN_EBP : STD_LOGIC_VECTOR (31 downto 0) := "1111111111" & "1001111011001111111111";
    constant Bits_Controle_MUX_OUT : STD_LOGIC_VECTOR (31 downto 0) := "1111111111" & "0000000001010000000000";
    constant Code_Instruction_CALL : STD_LOGIC_VECTOR (4 downto 0) := "10011";
    constant Code_Instruction_RET : STD_LOGIC_VECTOR (4 downto 0) := "10100";
    
    constant CNULL : STD_LOGIC_VECTOR (4 downto 0) := "00000";
    constant CWR : STD_LOGIC_VECTOR (4 downto 0) := "01011";
    constant CCALL : STD_LOGIC_VECTOR (4 downto 0) := "10011";
    constant CRET : STD_LOGIC_VECTOR (4 downto 0) := "10100";
    constant C0 : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
    constant C1 : STD_LOGIC_VECTOR (7 downto 0) := "00000001";
    constant C2 : STD_LOGIC_VECTOR (7 downto 0) := "00000010";
    constant C3 : STD_LOGIC_VECTOR (7 downto 0) := "00000011";
    constant C5 : STD_LOGIC_VECTOR (7 downto 0) := "00000101";
    constant C7 : STD_LOGIC_VECTOR (7 downto 0) := "00000111";
    constant C12 : STD_LOGIC_VECTOR (7 downto 0) := "00001100";
    constant C36 : STD_LOGIC_VECTOR (7 downto 0) := "00100100";
    constant C54 : STD_LOGIC_VECTOR (7 downto 0) := "00110110";
    constant C77 : STD_LOGIC_VECTOR (7 downto 0) := "01001101";
    constant C100 : STD_LOGIC_VECTOR (7 downto 0) := "01100100";
    
    constant CLK_period : time := 10 ns;
    
begin

    instance : Etage4_Memoire
    generic map( Nb_bits => 8,
                 Mem_size => 16,
                 Adresse_mem_size => 4,
                 Instruction_bus_size => 5,
                 Bits_Controle_LC => Bits_Controle_LC,
                 Bits_Controle_MUX_IN => Bits_Controle_MUX_IN,
                 Bits_Controle_MUX_IN_EBP => Bits_Controle_MUX_IN_EBP,
                 Bits_Controle_MUX_OUT => Bits_Controle_MUX_OUT,
                 Code_Instruction_CALL => Code_Instruction_CALL,
                 Code_Instruction_RET => Code_Instruction_RET)
    port map(    CLK => my_CLK,
                 RST => my_RST,
                 IN_A => my_IN_A,
                 IN_B => my_IN_B,
                 IN_Instruction => my_IN_Instruction,
                 OUT_A => my_OUT_A,
                 OUT_B => my_OUT_B,
                 OUT_Instruction => my_OUT_Instruction,
                 OUT_AddrRetour => my_OUT_AddrRetour);
    
    CLK_process :process
    begin
        my_CLK <= '0';
        wait for CLK_period/2;
        my_CLK <= '1';
        wait for CLK_period/2;
    end process;
    
    process 
    begin     
        my_IN_A <= C0 after 0 ns, C0  after 5 ns, C1 after 15 ns, C2 after 25 ns, C77 after 35 ns, C0  after 45 ns, C54 after 55 ns, C0   after 65 ns, C0 after 75 ns;
        my_IN_B <= C0 after 0 ns, C36 after 5 ns, C5 after 15 ns, C7 after 25 ns, C3  after 35 ns, C12 after 45 ns, C1  after 55 ns, C100 after 65 ns, C0 after 75 ns;
        my_IN_Instruction <= CNULL after 0 ns, CWR after 5 ns, CWR after 15 ns, CWR after 25 ns, CCALL  after 35 ns, CWR after 45 ns, CCALL after 55 ns, CWR after 65 ns, CRET after 75 ns, CRET after 85 ns, CNULL after 95 ns;
        my_RST <= '0' after 125 ns;
        wait;
    end process;    
end Behavioral;